#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.db.cache`` -- Cache Helpers
"""

from sqlalchemy.orm import joinedload

import edbob
from edbob.util import requires_impl

import rattail


class DataCacher(edbob.Object):

    def __init__(self, session=None, **kwargs):
        super(DataCacher, self).__init__(session=session, **kwargs)

    @property
    @requires_impl(is_property=True)
    def class_(self):
        pass
    
    @property
    def name(self):
        return self.class_.__name__

    def query(self):
        return self.session.query(self.class_)

    def get_cache(self, progress):
        self.instances = {}

        query = self.query()
        count = query.count()
        if not count:
            return self.instances
        
        prog = None
        # if progress:
        #     prog = progress("Reading %s records into cache" % self.name, count)

        cancel = False
        for i, instance in enumerate(query, 1):
            self.cache_instance(instance)
            if prog and not prog.update(i):
                cancel = True
                break

        if prog:
            prog.destroy()

        if cancel:
            session.close()
            return None
        return self.instances


class DepartmentCacher(DataCacher):

    class_ = rattail.Department

    def cache_instance(self, dept):
        self.instances[dept.number] = dept


class SubdepartmentCacher(DataCacher):

    class_ = rattail.Subdepartment

    def cache_instance(self, subdept):
        self.instances[subdept.number] = subdept


class BrandCacher(DataCacher):

    class_ = rattail.Brand

    def cache_instance(self, brand):
        self.instances[brand.name] = brand


class VendorCacher(DataCacher):

    class_ = rattail.Vendor

    def cache_instance(self, vend):
        self.instances[vend.id] = vend


class ProductCacher(DataCacher):

    class_ = rattail.Product
    with_costs = False

    def query(self):
        q = self.session.query(rattail.Product)
        if self.with_costs:
            q = q.options(joinedload(rattail.Product.cost))
        return q

    def cache_instance(self, prod):
        self.instances[prod.upc] = prod


def get_product_cache(session, with_costs=False, progress=None):
    """
    Cache the full product set by UPC.

    Returns a dictionary of all existing products, keyed by
    :attr:`rattail.Product.upc`.
    """

    cacher = ProductCacher(session=session, with_costs=with_costs)
    return cacher.get_cache(progress)


class CustomerGroupCacher(DataCacher):

    class_ = rattail.CustomerGroup

    def cache_instance(self, group):
        self.instances[group.id] = group


class CustomerCacher(DataCacher):

    class_ = rattail.Customer

    def cache_instance(self, customer):
        self.instances[customer.id] = customer



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.filemon`` -- Windows File Monitor
"""

from edbob.filemon.win32 import FileMonitorService


class RattailFileMonitor(FileMonitorService):

    _svc_name_ = "RattailFileMonitor"
    _svc_display_name_ = "Rattail : File Monitoring Service"

    appname = 'rattail'


if __name__ == '__main__':
    import win32serviceutil
    win32serviceutil.HandleCommandLine(RattailFileMonitor)



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.win32`` -- Windows Platform Utilities
"""


def process_is_elevated():
    """
    Check if the current process is running with an "elevated token."

    This is meant to indicate whether administrative privileges are in effect.
    Returns a boolean value.
    """

    from win32api import GetCurrentProcess
    from win32security import OpenProcessToken, GetTokenInformation, TokenElevation
    from win32con import TOKEN_READ

    hProcess = GetCurrentProcess()
    hToken = OpenProcessToken(hProcess, TOKEN_READ)
    elevated = GetTokenInformation(hToken, TokenElevation)
    return bool(elevated)


def require_elevation():
    """
    Exit properly if the current process does not have an "elevated token."

    If the result of :func:`process_is_elevated()` is ``False``, this function
    will print a brief message to ``sys.stderr`` and exit with the error code
    recommended by Microsoft for this scenario.
    """

    import sys
    from winerror import ERROR_ELEVATION_REQUIRED

    if process_is_elevated():
        return

    sys.stderr.write("This command requires administrative privileges.\n")
    sys.exit(ERROR_ELEVATION_REQUIRED)



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.files`` -- File Utilities

This module contains various utility functions for use with the filesystem.
"""

import os.path
import shutil
import lockfile
from datetime import datetime

import pkg_resources


def count_lines(path):
    """
    Counts the number of lines in a text file.  Some attempt is made to ensure
    cross-platform compatibility.

    :param path: Path to the file.
    :type path: string

    :returns: Number of lines in the file.
    :rtype: integer
    """

    f = open(path, 'rb')
    lines = f.read().count('\n') + 1
    f.close()
    return lines


def creation_time(path):
    """
    Returns the "naive" (i.e. not timezone-aware) creation timestamp for a
    file.

    :param path: Path to the file.
    :type path: string

    :returns: The creation timestamp for the file.
    :rtype: ``datetime.datetime`` instance
    """

    time = os.path.getctime(path)
    return datetime.fromtimestamp(time)


def locking_copy(src, dst):
    """
    Implements a "locking" version of ``shutil.copy()``.

    This function exists to provide a more atomic method for copying a file
    into a folder which is being watched by a file monitor.  The assumption is
    that the monitor is configured to expect "locks" and therefore only process
    files once they have had their locks removed.

    :param src: Path to the source file.
    :type src: string

    :param dst: Path to the destination file (or directory).
    :type dst: string

    :returns: ``None``
    """

    if os.path.isdir(dst):
        fn = os.path.basename(src)
        dst = os.path.join(dst, fn)

    with lockfile.FileLock(dst):
        shutil.copy(src, dst)


def resource_path(path):
    """
    Obtain a resource file path, extracting the resource and/or coercing the
    path as necessary.

    :param path: May be either a package resource specifier, or a regular file
       path.
    :type path: string

    :returns: Absolute file path to the resource.
    :rtype: string

    If ``path`` is a package resource specifier, and the package containing it
    is a zipped egg, then the resource will be extracted and the resultant
    filename will be returned.
    """

    if not os.path.isabs(path) and ':' in path:
        return pkg_resources.resource_filename(*path.split(':'))
    return path



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.db.batches.types`` -- Batch Types
"""

import datetime
import pkg_resources

from edbob.time import local_time

from rattail.exceptions import BatchTypeNotFound


__all__ = ['get_batch_type', 'BatchType']


batch_types = None


def get_batch_type(name):
    global batch_types
    if batch_types is None:
        batch_types = {}
        for entrypoint in pkg_resources.iter_entry_points('rattail.batches.types'):
            batch_types[entrypoint.name] = entrypoint.load()
    if name in batch_types:
        return batch_types[name]()
    raise BatchTypeNotFound(name)


class BatchType(object):

    name = None
    description = None
    source = None
    destination = None
    action_type = None

    purge_date_offset = None

    def initialize(self, batch):
        batch.provider = self.name
        batch.description = self.description
        batch.source = self.source
        batch.destination = self.destination
        batch.action_type = self.action_type
        self.set_purge_date(batch)
        self.add_columns(batch)

    def set_purge_date(self, batch):
        if self.purge_date_offset is not None:
            today = local_time().date()
            purge_offset = datetime.timedelta(days=self.purge_date_offset)
            batch.purge = today + purge_offset

    def add_columns(self, batch):
        raise NotImplementedError


# class LabelsBatchType(BatchType):

#     name = 'labels'
#     description = "Print Labels"

#     def add_columns(self, batch):
#         batch.add_column('F01')
#         batch.add_column('F155')
#         batch.add_column('F02')
#         batch.add_column('F22', display_name="Size")
#         batch.add_column('F95', display_name="Label")
#         batch.add_column('F94', display_name="Quantity")



#!/usr/bin/env python
# -*- coding: utf-8  -*-
################################################################################
#
#  Rattail -- Retail Software Framework
#  Copyright © 2010-2012 Lance Edgar
#
#  This file is part of Rattail.
#
#  Rattail is free software: you can redistribute it and/or modify it under the
#  terms of the GNU Affero General Public License as published by the Free
#  Software Foundation, either version 3 of the License, or (at your option)
#  any later version.
#
#  Rattail is distributed in the hope that it will be useful, but WITHOUT ANY
#  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
#  FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public License for
#  more details.
#
#  You should have received a copy of the GNU Affero General Public License
#  along with Rattail.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

"""
``rattail.csvutil`` -- CSV File Utilities

Contains various utilities relating to CSV file processing.

.. note::
   This module is named ``csvutil`` instead of ``csv`` primarily as a
   workaround to the problem of ``PythonService.exe`` insisting on doing
   relative imports.
"""

import csv


class DictWriter(csv.DictWriter):
    """
    Convenience class which derives from the standard ``csv.DictWriter``.  This
    currently exists only to provide the :meth:`writeheader()` method.
    """

    def writeheader(self):
        """
        Provide a ``writeheader()`` method in case the ``csv`` library in use
        doesn't (i.e. for Python < 2.7).
        """

        if hasattr(csv.DictWriter, 'writeheader'):
            return csv.DictWriter.writeheader(self)
        self.writerow(self.fieldnames)
